/*  This file is part of the KDE Libraries
 *  Copyright (C) 1998 Thomas Tanghus (tanghus@earthling.net)
 *  Additions 1999-2000 by Espen Sand (espen@kde.org)
 *                      by Holger Freyther <freyther@kde.org>
 *            2005-2006 by Olivier Goffart (ogoffart at kde.org)
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this library; see the file COPYING.LIB.  If not, write to
 *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 */

#include <kconfig.h>
#include <kapplication.h>
#include <kdialog.h>
#include <kwhatsthismanager_p.h>
#include <kdebug.h>
#include <kstaticdeleter.h>

#include <qlayout.h>
#include <qobject.h>
#include <qpointer.h>
#include <qlineedit.h>
#include <q3valuelist.h>
#include <qtimer.h>
#include <qcursor.h>
#include <QKeyEvent>
#include <QDesktopWidget>

#include <stdlib.h>

#ifdef KDE3_SUPPORT
#include <q3grid.h>
#endif

#include <qlayout.h>
#include <qtimer.h>
#include <QKeyEvent>
#include <QHideEvent>
#include <QDesktopWidget>
#include <QWhatsThis>
#include <QApplication>

#include <ktoolinvocation.h>
#include <klocale.h>
#include <kconfig.h>
#include <kiconloader.h>
#include <kglobal.h>
#include <kglobalsettings.h>
#include <kseparator.h>
#include <kurllabel.h>
#include <kdebug.h>
#include <khbox.h>
#include <kvbox.h>

#include "kdialogbase_priv.h"



#include "kdialogbase_priv.h"
#include "kdialogbase_priv.moc"



#include "config.h"
#ifdef Q_WS_X11
#include <qx11info_x11.h>
#include <netwm.h>
#endif

const int KDialog::mMarginSize = 11;
const int KDialog::mSpacingSize = 6;
KDialogTile *KDialog::mTile = 0;

int KDialogButton::id()
{
    return mKey;
}

/**
 * @internal
 */
namespace
{
struct SButton
{
  SButton()
  {
    box = 0;
    mask = 0;
    style = 0;
  }

  KPushButton *append( int key, const KGuiItem &item );

  void resize( bool sameWidth, int margin, int spacing, int orientation );

  KPushButton *button( int key );

  QWidget *box;
  int mask;
  int style;
  QList<KDialogButton*> list;
};
}// namespace

class KDialog::KDialogPrivate {
public:
    KDialogPrivate() : bDetails(false), bFixed(false), bSettingDetails(false), detailsWidget(0), 
        mTopLayout(0), mMainWidget(0), mUrlHelp(0), mJanus(0), mActionSep(0),
        mIsActivated(false), mShowTile(false), 
        mButtonOrientation(Qt::Horizontal)  { }

    bool bDetails;
    bool bFixed;
    bool bSettingDetails;
    QWidget *detailsWidget;
    QSize incSize;
    QSize minSize;
    QString detailsButton;
    SButton mButton;
/////////
    QBoxLayout   *mTopLayout;
    QWidget      *mMainWidget;
    KURLLabel    *mUrlHelp;
    KJanusWidget *mJanus;
    KSeparator   *mActionSep;

    bool mIsActivated;

    QString mAnchor;
    QString mHelpApp;
    QString mHelpLinkText;

    bool   mShowTile;

    int  mButtonOrientation;
    ButtonCode mEscapeButton;
};


#ifdef KDE3_SUPPORT
KDialog::KDialog(QWidget *parent, const char *name, bool modal, Qt::WFlags f)
  : QDialog(parent, name, modal, f), d(new KDialogPrivate)
{
    KWhatsThisManager::init ();
}
#endif


KDialog::KDialog( QWidget *parent, const QString &caption  ,
		int buttonMask , int dialogFace ,  Qt::WFlags f  ,
		const KGuiItem &user1,  const KGuiItem &user2, const KGuiItem &user3 )
    :  QDialog( parent, f ),  d(new KDialogPrivate)
{
  setCaption( caption );
  KWhatsThisManager::init();

  makeRelay();
  connect( this, SIGNAL(layoutHintChanged()), SLOT(updateGeometry()) );

  d->mJanus = new KJanusWidget( this, dialogFace );
  connect(d->mJanus, SIGNAL(currentPageChanged(QWidget *)),
          this, SIGNAL(currentPageChanged(QWidget *)));

  if( !d->mJanus || !d->mJanus->isValid() ) { return; }

  makeButtonBox( buttonMask, user1, user2, user3 );

  d->mIsActivated = true;
  setupLayout();
}


KDialog::~KDialog()
{
  delete d;
}



void KDialog::makeButtonBox( int buttonMask ,
				 const KGuiItem &user1, const KGuiItem &user2,
				 const KGuiItem &user3 )
{
  if( buttonMask == 0 )
  {
    d->mButton.box = 0;
    return; // When we want no button box
  }

  if( buttonMask & Cancel ) { buttonMask &= ~Close; }
  if( buttonMask & Apply ) { buttonMask &= ~Try; }
  if( buttonMask & Details ) { buttonMask &= ~Default; }

   d->mEscapeButton = (buttonMask&Cancel) ? Cancel : Close;

  d->mButton.box = new QWidget( this );

  d->mButton.mask = buttonMask;
  if( d->mButton.mask & Help )
  {
    KPushButton *pb = d->mButton.append( Help, KStdGuiItem::help() );

    connect( pb, SIGNAL(clicked()), SLOT(slotHelp()) );
  }
  if( d->mButton.mask & Default )
  {
    KPushButton *pb = d->mButton.append( Default, KStdGuiItem::defaults() );

    connect( pb, SIGNAL(clicked()), SLOT(slotDefault()) );
  }
  if( d->mButton.mask & Details )
  {
    KPushButton *pb = d->mButton.append( Details, QString() );
    connect( pb, SIGNAL(clicked()), SLOT(slotDetails()) );
    setDetails(false);
  }
  if( d->mButton.mask & User3 )
  {
    KPushButton *pb = d->mButton.append( User3, user3 );
    connect( pb, SIGNAL(clicked()), SLOT(slotUser3()) );
  }
  if( d->mButton.mask & User2 )
  {
    KPushButton *pb = d->mButton.append( User2, user2 );
      connect( pb, SIGNAL(clicked()), this, SLOT(slotUser2()) );
  }
  if( d->mButton.mask & User1 )
  {
    KPushButton *pb = d->mButton.append( User1, user1 );
      connect( pb, SIGNAL(clicked()), SLOT(slotUser1()) );
  }
  if( d->mButton.mask & Ok )
  {
    KPushButton *pb = d->mButton.append( Ok, KStdGuiItem::ok() );
    connect( pb, SIGNAL(clicked()), SLOT(slotOk()) );
  }
  if( d->mButton.mask & Apply )
  {
    KPushButton *pb = d->mButton.append( Apply, KStdGuiItem::apply() );
    connect( pb, SIGNAL(clicked()), SLOT(slotApply()) );
    connect( pb, SIGNAL(clicked()), SLOT(applyPressed()) );
  }
  if( d->mButton.mask & Try )
  {
    KPushButton *pb = d->mButton.append( Try,
                           i18n( "&Try" ) );
    connect( pb, SIGNAL(clicked()), SLOT(slotTry()) );
  }
  if( d->mButton.mask & Cancel )
  {
    KPushButton *pb = d->mButton.append( Cancel, KStdGuiItem::cancel() );
    connect( pb, SIGNAL(clicked()), SLOT(slotCancel()) );
  }
  if( d->mButton.mask & Close )
  {
    KPushButton *pb = d->mButton.append( Close, KStdGuiItem::close() );
    connect( pb, SIGNAL(clicked()), SLOT(slotClose()) );
  }

  setButtonStyle( KGlobalSettings::buttonLayout() );
}



void KDialog::setButtonStyle( int style )
{
  if( !d->mButton.box )
  {
    return;
  }

  if( style < 0 || style > ActionStyleMAX ) { style = ActionStyle0; }
  d->mButton.style = style;

  const int *layout;
  int layoutMax = 0;
/*  if (d->mMessageBoxMode)
  {
    static const int layoutRule[5][6] =
    {
      {Details,Stretch,User2|Stretch,User1|Stretch,Cancel|Stretch, Details|Filler},
      {Details,Stretch,User2|Stretch,User1|Stretch,Cancel|Stretch, Details|Filler},
      {Details,Stretch,User2|Stretch,User1|Stretch,Cancel|Stretch, Details|Filler},
      {Details|Filler,Stretch,Cancel|Stretch,User2|Stretch,User1|Stretch,Details},
      {Details|Filler,Stretch,Cancel|Stretch,User2|Stretch,User1|Stretch,Details}
    };
    layoutMax = 6;
    layout = layoutRule[ d->mButton.style ];
  }
  else*/ if (d->mButtonOrientation == Qt::Horizontal)
  {
    static const int layoutRule[5][10] =
    {
      {Details,Help,Default,Stretch,User3,User2,User1,Ok,Apply|Try,Cancel|Close},
      {Details,Help,Default,Stretch,User3,User2,User1,Cancel|Close,Apply|Try,Ok},
      {Details,Help,Default,Stretch,User3,User2,User1,Apply|Try,Cancel|Close,Ok},
      {Ok,Apply|Try,Cancel|Close,User3,User2,User1,Stretch,Default,Help,Details},
      {Ok,Cancel|Close,Apply|Try,User3,User2,User1,Stretch,Default,Help,Details}
    };
    layoutMax = 10;
    layout = layoutRule[ d->mButton.style ];
  }
  else
  {
    static const int layoutRule[5][10] =
    {
      {Ok,Apply|Try,User1,User2,User3,Stretch,Default,Cancel|Close,Help, Details},
      //{Ok,Apply|Try,Cancel|Close,User1,User2,User3,Stretch, Default,Help, Details},
      {Details,Help,Default,Stretch,User3,User2,User1,Cancel|Close,Apply|Try,Ok},
      {Details,Help,Default,Stretch,User3,User2,User1,Apply|Try,Cancel|Close,Ok},
      {Ok,Apply|Try,Cancel|Close,User3,User2,User1,Stretch,Default,Help,Details},
      {Ok,Cancel|Close,Apply|Try,User3,User2,User1,Stretch,Default,Help,Details}
    };
    layoutMax = 10;
    layout = layoutRule[ d->mButton.style ];
  }

  if( d->mButton.box->layout() )
  {
    delete d->mButton.box->layout();
  }

  QBoxLayout *lay;
  if( d->mButtonOrientation == Qt::Horizontal )
  {
    lay = new QBoxLayout( d->mButton.box, QBoxLayout::LeftToRight, 0,
			  spacingHint());
  }
  else
  {
    lay = new QBoxLayout( d->mButton.box, QBoxLayout::TopToBottom, 0,
			  spacingHint());
  }

  int numButton = 0;
  QPushButton *prevButton = 0;
  QPushButton *newButton;

  for( int i=0; i<layoutMax; ++i )
  {
    if(((ButtonCode) layout[i]) == Stretch) // Unconditional Stretch
    {
      lay->addStretch(1);
      continue;
    }
    else if (layout[i] & Filler) // Conditional space
    {
      if (d->mButton.mask & layout[i])
      {
        newButton = actionButton( (ButtonCode) (layout[i] & ~(Stretch | Filler)));
        if (newButton)
           lay->addSpacing(newButton->sizeHint().width());
      }
      continue;
    }
    else if( d->mButton.mask & Help & layout[i] )
    {
      newButton = actionButton( Help );
      lay->addWidget( newButton ); ++numButton;
    }
    else if( d->mButton.mask & Default & layout[i] )
    {
      newButton = actionButton( Default );
      lay->addWidget( newButton ); ++numButton;
    }
    else if( d->mButton.mask & User3 & layout[i] )
    {
      newButton = actionButton( User3 );
      lay->addWidget( newButton ); ++numButton;
    }
    else if( d->mButton.mask & User2 & layout[i] )
    {
      newButton = actionButton( User2 );
      lay->addWidget( newButton ); ++numButton;
    }
    else if( d->mButton.mask & User1 & layout[i] )
    {
      newButton = actionButton( User1 );
      lay->addWidget( newButton ); ++numButton;
    }
    else if( d->mButton.mask & Ok & layout[i] )
    {
      newButton = actionButton( Ok );
      lay->addWidget( newButton ); ++numButton;
    }
    else if( d->mButton.mask & Apply & layout[i] )
    {
      newButton = actionButton( Apply );
      lay->addWidget( newButton ); ++numButton;
    }
    else if( d->mButton.mask & Try & layout[i] )
    {
      newButton = actionButton( Try );
      lay->addWidget( newButton ); ++numButton;
    }
    else if( d->mButton.mask & Cancel & layout[i] )
    {
      newButton = actionButton( Cancel );
      lay->addWidget( newButton ); ++numButton;
    }
    else if( d->mButton.mask & Close & layout[i] )
    {
      newButton = actionButton( Close );
      lay->addWidget( newButton ); ++numButton;
    }
    else if( d->mButton.mask & Details & layout[i] )
    {
      newButton = actionButton( Details );
      lay->addWidget( newButton ); ++numButton;
    }
    else
    {
      continue;
    }

    // Add conditional stretch (Only added if a button was added)
    if(layout[i] & Stretch)
    {
      lay->addStretch(1);
    }

    if( prevButton )
    {
      setTabOrder( prevButton, newButton );
    }
    prevButton = newButton;
  }

  d->mButton.resize( false, 0, spacingHint(), d->mButtonOrientation );
}



void KDialog::setButtonBoxOrientation( int orientation )
{
  if( d->mButtonOrientation != orientation )
  {
    d->mButtonOrientation = orientation;
    if( d->mActionSep )
    {
      d->mActionSep->setOrientation( d->mButtonOrientation == Qt::Horizontal ?
				  Qt::Horizontal : Qt::Vertical );
    }
    if( d->mButtonOrientation == Qt::Vertical )
    {
      enableLinkedHelp(false); // 2000-06-18 Espen: No support for this yet.
    }
    setupLayout();
    setButtonStyle( d->mButton.style );
  }
}


void KDialog::setEscapeButton( ButtonCode id )
{
  d->mEscapeButton = id;
}

void KDialog::setDefaultButton( ButtonCode defaultButton )
{
  if( defaultButton != NoDefault )
  {
    QPushButton *pb = actionButton( defaultButton );
    if( pb )
    {
      setButtonFocus( pb, true, false );
    }
  }
}



void KDialog::adjustSize()
{
//  if (layout())
//     layout()->activate();
  if( d->bFixed )
    setFixedSize( sizeHint() );
  else
    resize( sizeHint() );
}

QSize KDialog::sizeHint() const
{
   return d->minSize.expandedTo( minimumSizeHint() ) + d->incSize;
}

QSize KDialog::minimumSizeHint() const
{
  const int m = marginHint();
  const int s = spacingHint();

  const QSize zeroByS(0, s);

  QSize s1(0,0);
  QSize s2(0,0);

  //
  // Url help area
  //
  if( d->mUrlHelp )
  {
    s2 = d->mUrlHelp->minimumSize() + zeroByS;
  }
  s1.rwidth()   = qMax( s1.rwidth(), s2.rwidth() );
  s1.rheight() += s2.rheight();

  //
  // User widget
  //
  if( d->mJanus )
  {
    s2 = d->mJanus->minimumSizeHint() + zeroByS;
  }
  else if( d->mMainWidget )
  {
    s2 = d->mMainWidget->sizeHint() + zeroByS;
    s2 = s2.expandedTo( d->mMainWidget->minimumSize() );
    s2 = s2.expandedTo( d->mMainWidget->minimumSizeHint() );
    if( s2.isEmpty() )
    {
      s2 = QSize( 100, 100+s );
    }
  }
  else
  {
    s2 = QSize( 100, 100+s );
  }
  s1.rwidth()  = qMax( s1.rwidth(), s2.rwidth() );
  s1.rheight() += s2.rheight();

  if (d->detailsWidget && d->bDetails)
  {
    s2 = d->detailsWidget->sizeHint() + zeroByS;
    s2 = s2.expandedTo( d->detailsWidget->minimumSize() );
    s2 = s2.expandedTo( d->detailsWidget->minimumSizeHint() );
    s1.rwidth()  = qMax( s1.rwidth(), s2.rwidth() );
    s1.rheight() += s2.rheight();
  }

  //
  // Button separator
  //
  if( d->mActionSep )
  {
    s1.rheight() += d->mActionSep->minimumSize().height() + s;
  }

  //
  // The button box
  //
  if( d->mButton.box )
  {
    s2 = d->mButton.box->minimumSize();
    if( d->mButtonOrientation == Qt::Horizontal )
    {
      s1.rwidth()   = qMax( s1.rwidth(), s2.rwidth() );
      s1.rheight() += s2.rheight();
    }
    else
    {
      s1.rwidth() += s2.rwidth();
      s1.rheight() = qMax( s1.rheight(), s2.rheight() );
    }
  }

  //
  // Outer margings
  //
  s1.rheight() += 2*m;
  s1.rwidth()  += 2*m;

  return s1;
}

QFrame *KDialog::makeMainWidget()
{
  if( d->mJanus || d->mMainWidget )
  {
    printMakeMainWidgetError();
    return 0;
  }

  QFrame *mainWidget = new QFrame( this );
  setMainWidget( mainWidget );
  return mainWidget;
}


KVBox *KDialog::makeVBoxMainWidget()
{
  if( d->mJanus || d->mMainWidget )
  {
    printMakeMainWidgetError();
    return 0;
  }

  KVBox *mainWidget = new KVBox( this );
  mainWidget->setSpacing( spacingHint() );
  setMainWidget( mainWidget );
  return mainWidget;
}


KHBox *KDialog::makeHBoxMainWidget()
{
  if( d->mJanus || d->mMainWidget )
  {
    printMakeMainWidgetError();
    return 0;
  }

  KHBox *mainWidget = new KHBox( this );
  mainWidget->setSpacing( spacingHint() );
  setMainWidget( mainWidget );
  return mainWidget;
}

#ifdef KDE3_SUPPORT
Q3Grid *KDialog::makeGridMainWidget( int n, Qt::Orientation dir )
{
  if( d->mJanus || d->mMainWidget )
  {
    printMakeMainWidgetError();
    return 0;
  }

  Q3Grid *mainWidget = new Q3Grid( n, dir, this );
  mainWidget->setSpacing( spacingHint() );
  setMainWidget( mainWidget );
  return mainWidget;
}
#endif

void KDialog::printMakeMainWidgetError()
{
  if( d->mJanus )
  {
    kdDebug() << "makeMainWidget: Illegal mode (wrong constructor)" << endl;
  }
  else if( d->mMainWidget )
  {
    kdDebug() << "makeMainWidget: Main widget already defined" << endl;
  }
}


void KDialog::setMainWidget( QWidget *widget )
{
  if( d->mJanus )
  {
    if( d->mJanus->setSwallowedWidget(widget) )
    {
      d->mMainWidget = widget;
    }
  }
  else
  {
    d->mMainWidget = widget;
    if( d->mIsActivated )
    {
      setupLayout();
    }
  }
  if( d->mMainWidget != NULL )
  {
    /* There is no more QFocusData in Qt4, so i used QWidget::nextInFocusChain()
       instead - mattr */
    QWidget* prev = nextInFocusChain();
    foreach ( KDialogButton* button, d->mButton.list )
    {
      if( prev != button )
	setTabOrder( prev, button );
      prev = button;
    }
  }
}


QWidget *KDialog::mainWidget()
{
  return d->mMainWidget;
}

QFrame *KDialog::addPage( const QString &itemName, const QString &header,
			      const QPixmap &pixmap )
{
  return ( d->mJanus ? d->mJanus->addPage( itemName, header, pixmap ) : 0);
}

QFrame *KDialog::addPage( const QStringList &items, const QString &header,
			      const QPixmap &pixmap )
{
  return ( d->mJanus ? d->mJanus->addPage( items, header, pixmap ) : 0);
}


KVBox *KDialog::addVBoxPage( const QString &itemName,
				 const QString &header, const QPixmap &pixmap )
{
  return ( d->mJanus ? d->mJanus->addVBoxPage( itemName, header, pixmap) : 0);
}

KVBox *KDialog::addVBoxPage( const QStringList &items,
				 const QString &header, const QPixmap &pixmap )
{
  return ( d->mJanus ? d->mJanus->addVBoxPage( items, header, pixmap) : 0);
}


KHBox *KDialog::addHBoxPage( const QString &itemName,
				 const QString &header,
				 const QPixmap &pixmap )
{
  return ( d->mJanus ? d->mJanus->addHBoxPage( itemName, header, pixmap ) : 0);
}

KHBox *KDialog::addHBoxPage( const QStringList &items,
				 const QString &header,
				 const QPixmap &pixmap )
{
  return ( d->mJanus ? d->mJanus->addHBoxPage( items, header, pixmap ) : 0);
}

#ifdef KDE3_SUPPORT
Q3Grid *KDialog::addGridPage( int n, Qt::Orientation dir,
				 const QString &itemName,
				 const QString &header, const QPixmap &pixmap )
{
  return ( d->mJanus ? d->mJanus->addGridPage( n, dir, itemName, header, pixmap) : 0);
}

Q3Grid *KDialog::addGridPage( int n, Qt::Orientation dir,
				 const QStringList &items,
				 const QString &header, const QPixmap &pixmap )
{
  return ( d->mJanus ? d->mJanus->addGridPage( n, dir, items, header, pixmap) : 0);
}
#endif



//
// Grab QDialogs keypresses if non-modal.
//
void KDialog::keyPressEvent(QKeyEvent *e)
{
  if( e->state() == 0 )
  {
    if( e->key() == Qt::Key_F1 )
    {
      QPushButton *pb = actionButton( Help );
      if( pb )
      {
	pb->animateClick();
	e->accept();
	return;
      }
    }
    if( e->key() == Qt::Key_Escape )
    {
      QPushButton *pb = actionButton( d->mEscapeButton );
      if( pb )
      {
	pb->animateClick();
	e->accept();
	return;
      }

    }
  }
  else if( e->key() == Qt::Key_F1 && e->state() == Qt::ShiftModifier )
  {
    QWhatsThis::enterWhatsThisMode();
    e->accept();
    return;
  }

  // accept the dialog when Ctrl-Return is pressed
  else if ( e->state() == Qt::ControlModifier &&
            (e->key() == Qt::Key_Return || e->key() == Qt::Key_Enter) )
  {
    QPushButton *pb = actionButton( Ok );
    if ( pb )
    {
      pb->animateClick();
      e->accept();
      return;
    }
  }

  QDialog::keyPressEvent(e);
}


int KDialog::marginHint()
{
  return mMarginSize;
}


int KDialog::spacingHint()
{
  return mSpacingSize;
}


void KDialog::setCaption( const QString &_caption )
{
  QString caption = kapp ? kapp->makeStdCaption( _caption ) : _caption;
  setPlainCaption( caption );
}


void KDialog::setPlainCaption( const QString &caption )
{
  QDialog::setCaption( caption );

#ifdef Q_WS_X11
  NETWinInfo info( QX11Info::display(), winId(), QX11Info::appRootWindow(), 0 );
  info.setName( caption.toUtf8().data() );
#endif
}


void KDialog::resizeLayout( QWidget *w, int margin, int spacing )
{
  if( w->layout() )
  {
    resizeLayout( w->layout(), margin, spacing );
  }

  if ( w->children().count() > 0 )
  {
	  QList<QObject*> l = w->children();
	  foreach ( QObject *o, l ) {
		  if( o->isWidgetType() )
		  {
			  resizeLayout( (QWidget*)o, margin, spacing );
		  }
	  }
  }
}


void KDialog::resizeLayout( QLayout *lay, int margin, int spacing )
{
  QLayoutItem *child;
  int pos = 0;
  while ( (child = lay->itemAt(pos) ) )
  {
    if ( child->layout() )
      resizeLayout( child->layout(), margin, spacing );
    ++pos;
  }
  if( lay->layout() )
  {
    lay->layout()->setMargin( margin );
    lay->layout()->setSpacing( spacing );
  }
}

static QRect screenRect( QWidget *w, int screen )
{
  QDesktopWidget *desktop = QApplication::desktop();
  KConfig gc("kdeglobals", false, false);
  gc.setGroup("Windows");
  if (desktop->isVirtualDesktop() &&
      gc.readBoolEntry("XineramaEnabled", true) &&
      gc.readBoolEntry("XineramaPlacementEnabled", true)) {
    if ( screen < 0 || screen >= desktop->numScreens() ) {
      if ( screen == -1 ) {
        screen = desktop->primaryScreen();
      } else if ( screen == -3 ) {
        screen = desktop->screenNumber( QCursor::pos() );
      } else {
        screen = desktop->screenNumber( w );
      }
    }
    return desktop->screenGeometry(screen);
  } else {
    return desktop->geometry();
  }
}

void KDialog::centerOnScreen( QWidget *w, int screen )
{
  if ( !w )
    return;
  QRect r = screenRect( w, screen );

  w->move( r.center().x() - w->width()/2,
           r.center().y() - w->height()/2 );
}

bool KDialog::avoidArea( QWidget *w, const QRect& area, int screen )
{
  if ( !w )
    return false;
  QRect fg = w->frameGeometry();
  if ( !fg.intersects( area ) )
    return true; // nothing to do.

  QRect scr = screenRect( w, screen );
  QRect avoid( area ); // let's add some margin
  avoid.moveBy( -5, -5 );
  avoid.rRight() += 10;
  avoid.rBottom() += 10;

  if ( qMax( fg.top(), avoid.top() ) <= qMin( fg.bottom(), avoid.bottom() ) )
  {
    // We need to move the widget up or down
    int spaceAbove = qMax(0, avoid.top() - scr.top());
    int spaceBelow = qMax(0, scr.bottom() - avoid.bottom());
    if ( spaceAbove > spaceBelow ) // where's the biggest side?
      if ( fg.height() <= spaceAbove ) // big enough?
        fg.setY( avoid.top() - fg.height() );
      else
        return false;
    else
      if ( fg.height() <= spaceBelow ) // big enough?
        fg.setY( avoid.bottom() );
      else
        return false;
  }

  if ( qMax( fg.left(), avoid.left() ) <= qMin( fg.right(), avoid.right() ) )
  {
    // We need to move the widget left or right
    int spaceLeft = qMax(0, avoid.left() - scr.left());
    int spaceRight = qMax(0, scr.right() - avoid.right());
    if ( spaceLeft > spaceRight ) // where's the biggest side?
      if ( fg.width() <= spaceLeft ) // big enough?
        fg.setX( avoid.left() - fg.width() );
      else
        return false;
    else
      if ( fg.width() <= spaceRight ) // big enough?
        fg.setX( avoid.right() );
      else
        return false;
  }
  //kdDebug() << "Moving window to " << fg.x() << "," << fg.y() << endl;
  w->move(fg.x(), fg.y());
  return true;
}

































class KDialogQueuePrivate
{
public:
  QList< QPointer<QDialog> > queue;
  bool busy;
};

static KStaticDeleter<KDialogQueue> ksdkdq;

KDialogQueue *KDialogQueue::_self=0;

KDialogQueue* KDialogQueue::self()
{
   if (!_self)
      _self = ksdkdq.setObject(_self, new KDialogQueue);
   return _self;
}

KDialogQueue::KDialogQueue() : d(new KDialogQueuePrivate)
{
   d->busy = false;
}

KDialogQueue::~KDialogQueue()
{
   delete d;
   _self = 0;
}

// static
void KDialogQueue::queueDialog(QDialog *dialog)
{
   KDialogQueue *_this = self();
   _this->d->queue.append(dialog);
   QTimer::singleShot(0, _this, SLOT(slotShowQueuedDialog()));
}

void KDialogQueue::slotShowQueuedDialog()
{
   if (d->busy)
      return;
   QDialog *dialog;
   do {
       if(d->queue.isEmpty())
         return;
      dialog = d->queue.first();
      d->queue.pop_front();
   }
   while(!dialog);

   d->busy = true;
   dialog->exec();
   d->busy = false;
   delete dialog;

   if (!d->queue.isEmpty())
      QTimer::singleShot(20, this, SLOT(slotShowQueuedDialog()));
   else
      ksdkdq.destructObject(); // Suicide.
}

KPushButton *SButton::append( int key, const KGuiItem &item )
{
  KDialogButton *p = new KDialogButton( item, key, box );
  list.append( p );
  return p;
}

void SButton::resize( bool sameWidth, int margin,
    int spacing, int orientation )
{
  KDialogButton *p;
  int h = 0;
  int w = 0;
  int t = 0;

  foreach ( p, list )
  {
    const QSize s( p->sizeHint() );
    if( s.height() > h ) { h = s.height(); }
    if( s.width() > w ) { w = s.width(); }
  }

  if( orientation == Qt::Horizontal )
  {
    foreach ( p, list )
    {
      QSize s( p->sizeHint() );
      if( sameWidth ) { s.setWidth( w ); }
      p->setFixedWidth( s.width() );
      t += s.width() + spacing;
    }

    box->setMinimumHeight( margin*2 + h );
    box->setMinimumWidth( margin*2 + t - spacing );
  }
  else
  {
    // sameWidth has no effect here
    foreach ( p, list )
    {
      QSize s( p->sizeHint() );
      s.setWidth( w );
      p->setFixedSize( s );
      t += s.height() + spacing;
    }
    box->setMinimumHeight( margin*2 + t - spacing );
    box->setMinimumWidth( margin*2 + w );
  }
}

KPushButton *SButton::button( int key )
{
  foreach ( KDialogButton *p, list )
  {
    if( p->id() == key )
    {
      return p;
    }
  }
  return 0;
}


void KDialog::setupLayout()
{
  if( d->mTopLayout )
  {
    delete d->mTopLayout;
  }
  // d->mTopLayout = new QVBoxLayout( this, marginHint(), spacingHint() );


  if( d->mButtonOrientation == Qt::Horizontal )
  {
    d->mTopLayout = new QVBoxLayout( this );
    d->mTopLayout->setMargin(marginHint());
    d->mTopLayout->setSpacing(spacingHint());
  }
  else
  {
    d->mTopLayout = new QHBoxLayout( this);
    d->mTopLayout->setMargin(marginHint());
    d->mTopLayout->setSpacing(spacingHint());
  }

  if( d->mUrlHelp )
  {
    d->mTopLayout->addWidget( d->mUrlHelp, 0, Qt::AlignRight );
  }

  if( d->mJanus )
  {
    d->mTopLayout->addWidget( d->mJanus, 10 );
  }
  else if( d->mMainWidget )
  {
    d->mTopLayout->addWidget( d->mMainWidget, 10 );
  }

  if ( d->detailsWidget )
  {
    d->mTopLayout->addWidget( d->detailsWidget );
  }

  if( d->mActionSep )
  {
    d->mTopLayout->addWidget( d->mActionSep );
  }

  if( d->mButton.box )
  {
    d->mTopLayout->addWidget( d->mButton.box );
  }
}



void KDialog::makeRelay()
{
  if( mTile )
  {
    connect( mTile, SIGNAL(pixmapChanged()), SLOT(updateBackground()) );
    return;
  }

  mTile = new KDialogTile;
  if( mTile )
  {
    connect( mTile, SIGNAL(pixmapChanged()), SLOT(updateBackground()) );
    connect( qApp, SIGNAL(aboutToQuit()), mTile, SLOT(cleanup()) );
  }
}


void KDialog::enableButtonSeparator( bool state )
{
  if( state )
  {
    if( d->mActionSep )
    {
      return;
    }
    d->mActionSep = new KSeparator( this );
    d->mActionSep->setFocusPolicy(Qt::NoFocus);
    d->mActionSep->setOrientation( d->mButtonOrientation == Qt::Horizontal ?
				Qt::Horizontal : Qt::Vertical );
    d->mActionSep->show();
  }
  else
  {
    if( !d->mActionSep )
    {
      return;
    }
    delete d->mActionSep; d->mActionSep = 0;
  }

  if( d->mIsActivated )
  {
    setupLayout();
  }
}



QFrame *KDialog::plainPage()
{
	return ( d->mJanus ? d->mJanus->plainPage() : 0 );
}





void KDialog::disableResize()
{
  setFixedSize( sizeHint() );
}


void KDialog::setInitialSize( const QSize &s, bool noResize )
{
  d->minSize = s;
  d->bFixed = noResize;
  adjustSize();
}


void KDialog::incInitialSize( const QSize &s, bool noResize )
{
  d->incSize = s;
  d->bFixed = noResize;
  adjustSize();
}



KPushButton *KDialog::actionButton( ButtonCode id )
{
  return d->mButton.button(id);
}


void KDialog::enableButton( ButtonCode id, bool state )
{
  QPushButton *pb = actionButton( id );
  if( pb )
  {
    pb->setEnabled( state );
  }
}


void KDialog::enableButtonOK( bool state )
{
  enableButton( Ok, state );
}


void KDialog::enableButtonApply( bool state )
{
  enableButton( Apply, state );
}


void KDialog::enableButtonCancel( bool state )
{
  enableButton( Cancel, state );
}


void KDialog::showButton( ButtonCode id, bool state )
{
  QPushButton *pb = actionButton( id );
  if( pb )
  {
    state ? pb->show() : pb->hide();
  }
}

void KDialog::setButtonGuiItem( ButtonCode id, const KGuiItem &item )
{
  KPushButton *pb = static_cast<KPushButton *>( actionButton( id ) );
  if( !pb )
    return;

  pb->setGuiItem( item );
  d->mButton.resize( false, 0, spacingHint(), d->mButtonOrientation );
}


void KDialog::setButtonText( ButtonCode id, const QString &text )
{
  if (!d->bSettingDetails && (id == Details))
  {
    d->detailsButton = text;
    setDetails(d->bDetails);
    return;
  }
  QPushButton *pb = actionButton( id );
  if( pb )
  {
    pb->setText( text );
    d->mButton.resize( false, 0, spacingHint(), d->mButtonOrientation );
  }
}


void KDialog::setButtonTip( ButtonCode id, const QString &text )
{
  QPushButton *pb = actionButton( id );
  if( pb )
  {
    if (text.isEmpty())
      pb->setToolTip( QString() );
    else
      pb->setToolTip( text );
  }
}


void KDialog::setButtonWhatsThis( ButtonCode id, const QString &text )
{
  QPushButton *pb = actionButton( id );
  if( pb )
  {
    if (text.isEmpty())
      pb->setWhatsThis(QString::null);
    else
      pb->setWhatsThis(text );
  }
}


void KDialog::setButtonFocus( QPushButton *p,bool isDefault, bool isFocus )
{
  p->setDefault( isDefault );
  if( isFocus )
      p->setFocus();
}


void KDialog::setTreeListAutoResize( bool state )
{
  if( d->mJanus )
  {
    d->mJanus->setTreeListAutoResize( state );
  }
}

void KDialog::setShowIconsInTreeList(bool state)
{
  if( d->mJanus )
  {
    d->mJanus->setShowIconsInTreeList( state );
  }
}

void KDialog::setRootIsDecorated( bool state )
{
  if( d->mJanus )
  {
    d->mJanus->setRootIsDecorated( state );
  }
}

void KDialog::unfoldTreeList( bool persist )
{
  if( d->mJanus )
    d->mJanus->unfoldTreeList( persist );
}

void KDialog::addWidgetBelowList( QWidget * widget )
{
  if( d->mJanus )
    d->mJanus->addWidgetBelowList( widget );
}

void KDialog::addButtonBelowList( const QString & text, QObject * recv, const char * slot )
{
  if( d->mJanus )
    d->mJanus->addButtonBelowList( text, recv, slot );
}

void KDialog::addButtonBelowList( const KGuiItem & item, QObject * recv, const char * slot )
{
  if( d->mJanus )
    d->mJanus->addButtonBelowList( item, recv, slot );
}

void KDialog::setIconListAllVisible( bool state )
{
  if( d->mJanus )
  {
    d->mJanus->setIconListAllVisible( state );
  }
}


void KDialog::slotHelp()
{
  emit helpClicked();
  KToolInvocation::invokeHelp( d->mAnchor, d->mHelpApp );
}


void KDialog::slotDefault()
{
  emit defaultClicked();
}

void KDialog::slotDetails()
{
  setDetails(!d->bDetails);
}

void KDialog::setDetailsWidget(QWidget *detailsWidget)
{
  delete d->detailsWidget;
  d->detailsWidget = detailsWidget;
  if (d->detailsWidget->parentWidget() != this)
     d->detailsWidget->reparent(this, QPoint(0,0));
  d->detailsWidget->hide();
  if( d->mIsActivated )
  {
    setupLayout();
  }
  if (!d->bSettingDetails)
    setDetails(d->bDetails);
}

void KDialog::setDetails(bool showDetails)
{
  if (d->detailsButton.isEmpty())
     d->detailsButton = i18n("&Details");
  d->bSettingDetails = true;
  d->bDetails = showDetails;
  if (d->bDetails)
  {
     emit aboutToShowDetails();
     setButtonText(Details, d->detailsButton+ " <<");
     if (d->detailsWidget)
     {
        if (layout())
           layout()->setEnabled(false);
        adjustSize();
        d->detailsWidget->show();
        if (layout())
        {
           layout()->activate();
           layout()->setEnabled(true);
        }
     }
  }
  else
  {
     setButtonText(Details, d->detailsButton+" >>");
     if (d->detailsWidget)
     {
        d->detailsWidget->hide();
     }
     if (layout())
        layout()->activate();
     adjustSize();
  }
  d->bSettingDetails = false;
}

void KDialog::slotOk()
{
  emit okClicked();
  accept();
}


void KDialog::slotApply()
{
  emit applyClicked();
}


void KDialog::slotTry()
{
  emit tryClicked();
}


void KDialog::slotUser3()
{
  emit user3Clicked();
}


void KDialog::slotUser2()
{
  emit user2Clicked();
}


void KDialog::slotUser1()
{
  emit user1Clicked();
}


void KDialog::slotYes()
{
  emit yesClicked();
  done( Yes );
}


void KDialog::slotNo()
{
  emit noClicked();
  done( No );
}


void KDialog::slotCancel()
{
  emit cancelClicked();
  reject();
}


void KDialog::slotClose()
{
  emit closeClicked();
  reject();
}


void KDialog::helpClickedSlot( const QString & )
{
  slotHelp();
}



void KDialog::enableLinkedHelp( bool state )
{
  if( state )
  {
    if( d->mUrlHelp )
    {
      return;
    }

    d->mUrlHelp = new KURLLabel( this );
    d->mUrlHelp->setText( helpLinkText() );
    d->mUrlHelp->setFloat(true);
    d->mUrlHelp->setUnderline(true);
    if( d->mShowTile && mTile->get() )
    {
      d->mUrlHelp->setBackgroundPixmap(*mTile->get());
    }
    d->mUrlHelp->setMinimumHeight( fontMetrics().height() + marginHint() );
    connect(d->mUrlHelp,SIGNAL(leftClickedURL(const QString &)),
	    SLOT(helpClickedSlot(const QString &)));
    d->mUrlHelp->show();
  }
  else
  {
    if( !d->mUrlHelp )
    {
      return;
    }
    delete d->mUrlHelp; d->mUrlHelp = 0;
  }

  if( d->mIsActivated )
  {
    setupLayout();
  }
}


void KDialog::setHelp( const QString &anchor, const QString &appname )
{
  d->mAnchor  = anchor;
  d->mHelpApp = appname;
}


void KDialog::setHelpLinkText( const QString &text )
{
  d->mHelpLinkText = text;
  if( d->mUrlHelp )
  {
    d->mUrlHelp->setText( helpLinkText() );
  }
}


void KDialog::setFolderIcon(const QStringList &path, const QPixmap &pixmap)
{
  if (!d->mJanus)
    return;

  d->mJanus->setFolderIcon(path,pixmap);
}




bool KDialog::showPage( int index )
{
  return ( d->mJanus ? d->mJanus->showPage(index) : false);
}


int KDialog::activePageIndex() const
{
  return ( d->mJanus ? d->mJanus->activePageIndex() : -1);
}


int KDialog::pageIndex( QWidget *widget ) const
{
  return ( d->mJanus ? d->mJanus->pageIndex( widget) : -1);
}

QSize KDialog::calculateSize(int w, int h) const
{
  int ulx, uly, lrx, lry;
  ulx = marginHint();
  uly = marginHint();
  if( d->mUrlHelp  )
  {
    uly += d->mUrlHelp->minimumSize().height();
  }

  lrx = marginHint();
  lry = d->mButton.box ? d->mButton.box->minimumSize().height() : 0;
  if( d->mActionSep )
  {
    lry += d->mActionSep->minimumSize().height() + marginHint();
  }
  return QSize(ulx+w+lrx,uly+h+lry);
}


QString KDialog::helpLinkText() const
{
  return ( d->mHelpLinkText.isNull() ? i18n("Get help...") : d->mHelpLinkText );
}


void KDialog::updateGeometry()
{
  if( d->mTopLayout )
  {
    d->mTopLayout->setMargin( marginHint() );
    d->mTopLayout->setSpacing(spacingHint() );
  }
}





void KDialog::hideEvent( QHideEvent *ev )
{
  emit hidden();
  if (!ev->spontaneous())
  {
     emit finished();
  }
}



void KDialog::closeEvent( QCloseEvent *e )
{
    QPushButton *pb = actionButton( d->mEscapeButton );
    if( pb && isShown() ) {
	pb->animateClick();
    } else {
	QDialog::closeEvent( e );
    }
}

void KDialog::cancel()
{
    switch ( d->mEscapeButton ) {
    case Ok:
        slotOk();
        break;
    case User1: // == No
            slotUser1();
		break;
    case User2: // == Yes
            slotUser2();
        break;
    case User3:
        slotUser3();
        break;
    case Close:
        slotClose();
        break;
    case Cancel:
    default:
	slotCancel();
    }
}

bool KDialog::haveBackgroundTile()
{
  return ( !mTile || mTile->get() );
}

const QPixmap *KDialog::backgroundTile()
{
  return ( mTile ? mTile->get() : 0);
}


void KDialog::setBackgroundTile( const QPixmap *pix )
{
  if( mTile )
  {
    mTile->set( pix );
  }
}


void KDialog::updateBackground()
{
  if( !mTile || !mTile->get() )
  {
    QPixmap nullPixmap;
    setBackgroundPixmap(nullPixmap);
    if( d->mButton.box )
    {
      d->mButton.box->setBackgroundPixmap(nullPixmap);
      d->mButton.box->setBackgroundMode(Qt::PaletteBackground);
    }
    setBackgroundMode(Qt::PaletteBackground);
  }
  else
  {
    const QPixmap *pix = mTile->get();
    setBackgroundPixmap(*pix);
    if( d->mButton.box )
    {
      d->mButton.box->setBackgroundPixmap(*pix);
    }
    showTile( d->mShowTile );
  }
}


void KDialog::showTile( bool state )
{
  d->mShowTile = state;
  if( !d->mShowTile || !mTile || !mTile->get() )
  {
    setBackgroundMode(Qt::PaletteBackground);
    if( d->mButton.box )
    {
      d->mButton.box->setBackgroundMode(Qt::PaletteBackground);
    }
    if( d->mUrlHelp )
    {
      d->mUrlHelp->setBackgroundMode(Qt::PaletteBackground);
    }
  }
  else
  {
    const QPixmap *pix = mTile->get();
    setBackgroundPixmap(*pix);
    if( d->mButton.box )
    {
      d->mButton.box->setBackgroundPixmap(*pix);
    }
    if( d->mUrlHelp )
    {
      d->mUrlHelp->setBackgroundPixmap(*pix);
    }
  }
}

QSize KDialog::configDialogSize( const QString& groupName ) const
{
   return configDialogSize( *KGlobal::config(), groupName );
}


QSize KDialog::configDialogSize( KConfig& config,
				      const QString& groupName ) const
{
   int w, h;
   int scnum = QApplication::desktop()->screenNumber(parentWidget());
   QRect desk = QApplication::desktop()->screenGeometry(scnum);

   w = sizeHint().width();
   h = sizeHint().height();

   KConfigGroup cg(&config, groupName);
   w = cg.readNumEntry( QString::fromLatin1("Width %1").arg( desk.width()), w );
   h = cg.readNumEntry( QString::fromLatin1("Height %1").arg( desk.height()), h );

   return QSize( w, h );
}


void KDialog::saveDialogSize( const QString& groupName, bool global )
{
   saveDialogSize( *KGlobal::config(), groupName, global );
}


void KDialog::saveDialogSize( KConfig& config, const QString& groupName,
				      bool global ) const
{
   int scnum = QApplication::desktop()->screenNumber(parentWidget());
   QRect desk = QApplication::desktop()->screenGeometry(scnum);

   KConfigGroup cg(&config, groupName);
   QSize sizeToSave = size();

   cg.writeEntry( QString::fromLatin1("Width %1").arg( desk.width()),
		  QString::number( sizeToSave.width()), true, global);
   cg.writeEntry( QString::fromLatin1("Height %1").arg( desk.height()),
		  QString::number( sizeToSave.height()), true, global);
}


KDialogButton::KDialogButton( const KGuiItem &item, int key,
				      QWidget *parent )
  : KPushButton( item, parent ), mKey(key)
{
}




KDialogTile::KDialogTile( QObject *parent )
  : QObject( parent ), mPixmap(0)
{
}


KDialogTile::~KDialogTile()
{
  cleanup();
}


void KDialogTile::set( const QPixmap *pix )
{
  if( !pix )
  {
    cleanup();
  }
  else
  {
    if( !mPixmap )
    {
      mPixmap = new QPixmap(*pix);
    }
    else
    {
      *mPixmap = *pix;
    }
  }

  emit pixmapChanged();
}


const QPixmap *KDialogTile::get() const
{
  return mPixmap;
}


void KDialogTile::cleanup()
{
  delete mPixmap; mPixmap = 0;
}


void KDialog::virtual_hook( int, void* )
{ /*BASE::virtual_hook( id, data );*/ }

#include "kdialog.moc"
