/*
 *  This file is part of the KDE Libraries
 *  Copyright (C) 1999-2001 Mirko Boehm (mirko@kde.org) and
 *  Espen Sand (espen@kde.org)
 *  Holger Freyther <freyther@kde.org>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public License
 *  along with this library; see the file COPYING.LIB.  If not, write to
 *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301, USA.
 *
 */

#include "kdialogbase.h"

K3DialogBase::K3DialogBase( QWidget *parent, const char *name, bool modal,
			  const QString &caption, int buttonMask,
			  ButtonCode defaultButton, bool separator,
			  const KGuiItem &user1, const KGuiItem &user2,
			  const KGuiItem &user3 )
  :KDialog( parent, caption, buttonMask, Swallow, Qt::WStyle_DialogBorder , user1 , user2, user3 )
{
  setObjectName(name);
  setModal(modal);
  enableButtonSeparator(separator);
  setDefaultButton (defaultButton);
}

K3DialogBase::K3DialogBase( int dialogFace, const QString &caption,
			  int buttonMask, ButtonCode defaultButton,
			  QWidget *parent, const char *name, bool modal,
			  bool separator, const KGuiItem &user1,
			  const KGuiItem &user2, const KGuiItem &user3 )
  :KDialog( parent, caption, buttonMask, dialogFace, Qt::WStyle_DialogBorder , user1 , user2, user3 )
{
  setObjectName(name);
  setModal(modal);
  enableButtonSeparator(separator);
  setDefaultButton (defaultButton);
}

K3DialogBase::K3DialogBase(  KDialogBase::DialogType dialogFace, Qt::WFlags f, QWidget *parent, const char *name,
		          bool modal,
			  const QString &caption, int buttonMask,
			  ButtonCode defaultButton, bool separator,
			  const KGuiItem &user1, const KGuiItem &user2,
			  const KGuiItem &user3 )
  :KDialog( parent, caption, buttonMask, dialogFace, f , user1 , user2, user3 )
{
  setObjectName(name);
  setModal(modal);
  enableButtonSeparator(separator);
  setDefaultButton (defaultButton);
}

K3DialogBase::K3DialogBase( const QString &caption, int buttonMask,
			  ButtonCode defaultButton, ButtonCode escapeButton,
			  QWidget *parent, const char *name, bool modal,
			  bool separator, const KGuiItem &yes,
			  const KGuiItem &no, const KGuiItem &cancel )
  :KDialog( parent, caption, buttonMask, Swallow, Qt::WStyle_DialogBorder )
{
  setObjectName(name);
  setModal(modal);
  enableButtonSeparator(separator);
  setDefaultButton (defaultButton);
  setEscapeButton( escapeButton);
  setButtonGuiItem (Yes, yes);
  setButtonGuiItem (No, no);
  setButtonGuiItem (Cancel, cancel);
}

void
K3DialogBase::delayedDestruct()
{
   deleteLater();
}



void K3DialogBase::showButtonOK( bool state )
{
  showButton( Ok, state );
}


void K3DialogBase::showButtonApply( bool state )
{
  showButton( Apply, state );
}


void K3DialogBase::showButtonCancel( bool state )
{
  showButton( Cancel, state );
}


void K3DialogBase::setButtonOK( const KGuiItem &item /* = KStdGuiItem::ok() */ )
{
  setButtonGuiItem( Ok, item );
}


void K3DialogBase::setButtonApply( const KGuiItem &item /* = KStdGuiItem::apply() */ )
{
  setButtonGuiItem( Apply, item );
}


void K3DialogBase::setButtonCancel( const KGuiItem &item /* = KStdGuiItem::cancel() */ )
{
  setButtonGuiItem( Cancel, item );
}

